@@ -4,16 +4,21 @@ contents (think minification, compression).
 
 from __future__ import with_statement
 
-import os, subprocess
+import os
+import subprocess
 import inspect
 import shlex
 import tempfile
+from webassets import six
+from webassets.six.moves import map
+from webassets.six.moves import zip
 try:
     frozenset
 except NameError:
     from sets import ImmutableSet as frozenset
 from webassets.exceptions import FilterError
 from webassets.importlib import import_module
+from webassets.utils import hash_func
 
 
 __all__ = ('Filter', 'CallableFilter', 'get_filter', 'register_filter',
@@ -28,7 +33,7 @@ def freezedicts(obj):
     if isinstance(obj, (list, tuple)):
         return type(obj)([freezedicts(sub) for sub in obj])
     if isinstance(obj, dict):
-        return frozenset(obj.iteritems())
+        return frozenset(six.iteritems(obj))
     return obj
 
 
@@ -44,15 +49,16 @@ def smartsplit(string, sep):
     be done.
     """
     assert string is not None   # or shlex will read from stdin
-    # shlex fails miserably with unicode input
-    is_unicode = isinstance(sep, unicode)
-    if is_unicode:
-        string = string.encode('utf8')
+    if not six.PY3:
+        # On 2.6, shlex fails miserably with unicode input
+        is_unicode = isinstance(string, unicode)
+        if is_unicode:
+            string = string.encode('utf8')
     l = shlex.shlex(string, posix=True)
     l.whitespace += ','
     l.whitespace_split = True
     l.quotes = ''
-    if is_unicode:
+    if not six.PY3 and is_unicode:
         return map(lambda s: s.decode('utf8'), list(l))
     else:
         return list(l)
@@ -65,7 +71,8 @@ class option(tuple):
     See ``parse_options()`` and ``Filter.options``.
     """
     def __new__(cls, initarg, configvar=None, type=None):
-        if configvar is None:  # If only one argument given, it is the configvar
+        # If only one argument given, it is the configvar
+        if configvar is None:  
             configvar = initarg
             initarg = None
         return tuple.__new__(cls, (initarg, configvar, type))
@@ -129,7 +136,7 @@ class Filter(object):
     max_debug_level = False
 
     def __init__(self, **kwargs):
-        self.env = None
+        self.ctx = None
         self._options = parse_options(self.__class__.options)
 
         # Resolve options given directly to the filter. This
@@ -144,19 +151,16 @@ class Filter(object):
                 setattr(self, attribute, None)
         if kwargs:
             raise TypeError('got an unexpected keyword argument: %s' %
-                            kwargs.keys()[0])
+                            list(kwargs.keys())[0])
 
-    def __hash__(self):
-        return self.id()
-
-    def __cmp__(self, other):
+    def __eq__(self, other):
         if isinstance(other, Filter):
-            return cmp(self.id(), other.id())
+            return self.id() == other.id()
         return NotImplemented
 
-    def set_environment(self, env):
+    def set_context(self, ctx):
         """This is called before the filter is used."""
-        self.env = env
+        self.ctx = ctx
 
     def get_config(self, setting=False, env=None, require=True,
                    what='dependency', type=None):
@@ -194,12 +198,16 @@ class Filter(object):
 
         value = None
         if not setting is False:
-            value = self.env.config.get(setting, None)
+            value = self.ctx.get(setting, None)
 
         if value is None and not env is False:
             value = os.environ.get(env)
-            if value and type == list:
-                value = smartsplit(value, ',')
+            if value is not None:
+                if not six.PY3:
+                    # TODO: What charset should we use? What does Python 3 use?
+                    value = value.decode('utf8')
+                if type == list:
+                    value = smartsplit(value, ',')
 
         if value is None and require:
             err_msg = '%s was not found. Define a ' % what
@@ -233,7 +241,7 @@ class Filter(object):
         """
         # freezedicts() allows filters to return dict objects as part
         # of unique(), which are not per-se supported by hash().
-        return hash((self.name, freezedicts(self.unique()),))
+        return hash_func((self.name, freezedicts(self.unique()),))
 
     def setup(self):
         """Overwrite this to have the filter do initial setup work,
@@ -257,8 +265,8 @@ class Filter(object):
                 # No value specified for this filter instance ,
                 # specifically attempt to load it from the environment.
                 setattr(self, attribute,
-                    self.get_config(setting=configvar, require=False,
-                                    type=type))
+                        self.get_config(setting=configvar, require=False,
+                                        type=type))
 
     def input(self, _in, out, **kw):
         """Implement your actual filter here.
@@ -290,6 +298,23 @@ class Filter(object):
        Only one such filter is allowed.
        """
 
+    def get_additional_cache_keys(self, **kw):
+        """Additional cache keys dependent on keyword arguments.
+
+        If your filter's output is dependent on some or all of the
+        keyword arguments, you can return these arguments here as a list.
+        This will make sure the caching behavior is correct.
+
+        For example, the CSSRewrite filter depends not only on the
+        contents of the file it applies to, but also the output path
+        of the final file. If the CSSRewrite filter doesn't correctly
+        override this method, a certain output file with a certain base
+        directory might potentially get a CSSRewriten file from cache
+        that is meant for an output file in a different base directory.
+        """
+
+        return []
+
     # We just declared those for demonstration purposes
     del input
     del output
@@ -322,7 +347,57 @@ class CallableFilter(Filter):
         return self.callable(_in, out)
 
 
-class ExternalTool(Filter):
+class ExternalToolMetaclass(type):
+    def __new__(cls, name, bases, attrs):
+        # First, determine the method defined for this very class. We
+        # need to pop the ``method`` attribute from ``attrs``, so that we
+        # create the class without the argument; allowing us then to look
+        # at a ``method`` attribute that parents may have defined.
+        #
+        # method defaults to 'output' if argv is set, to "implement
+        # no default method" without an argv.
+        if not 'method' in attrs and 'argv' in attrs:
+            chosen = 'output'
+        else:
+            chosen = attrs.pop('method', False)
+
+        # Create the class first, since this helps us look at any
+        # method attributes defined in the parent hierarchy.
+        klass = type.__new__(cls, name, bases, attrs)
+        parent_method = getattr(klass, 'method', None)
+
+        # Assign the method argument that we initially popped again.
+        klass.method = chosen
+
+        try:
+            # Don't do anything for this class itself
+            ExternalTool
+        except NameError:
+            return klass
+
+        # If the class already has a method attribute, this indicates
+        # that a parent class already dealt with it and enabled/disabled
+        # the methods, and we won't again.
+        if parent_method is not None:
+            return klass
+
+        methods = ('output', 'input', 'open')
+
+        if chosen is not None:
+            assert not chosen or chosen in methods, \
+                '%s not a supported filter method' % chosen
+            # Disable those methods not chosen.
+            for m in methods:
+                if m != chosen:
+                    # setdefault = Don't override actual methods the
+                    # class has in fact provided itself.
+                    if not m in klass.__dict__:
+                        setattr(klass, m, None)
+
+        return klass
+
+
+class ExternalTool(six.with_metaclass(ExternalToolMetaclass, Filter)):
     """Subclass that helps creating filters that need to run an external
     program.
 
@@ -347,55 +422,6 @@ class ExternalTool(Filter):
     argv = []
     method = None
 
-    class __metaclass__(type):
-        def __new__(cls, name, bases, attrs):
-            # First, determine the method defined for this very class. We
-            # need to pop the ``method`` attribute from ``attrs``, so that we
-            # create the class without the argument; allowing us then to look
-            # at a ``method`` attribute that parents may have defined.
-            #
-            # method defaults to 'output' if argv is set, to "implement
-            # no default method" without an argv.
-            if not 'method' in attrs and 'argv' in attrs:
-                chosen = 'output'
-            else:
-                chosen = attrs.pop('method', False)
-
-            # Create the class first, since this helps us look at any
-            # method attributes defined in the parent hierarchy.
-            klass = type.__new__(cls, name, bases, attrs)
-            parent_method = getattr(klass, 'method', None)
-
-            # Assign the method argument that we initially popped again.
-            klass.method = chosen
-
-            try:
-                # Don't do anything for this class itself
-                ExternalTool
-            except NameError:
-                return klass
-
-            # If the class already has a method attribute, this indicates
-            # that a parent class already dealt with it and enabled/disabled
-            # the methods, and we won't again.
-            if parent_method is not None:
-                return klass
-
-            methods = ('output', 'input', 'open')
-
-            if chosen is not None:
-                assert not chosen or chosen in methods, \
-                    '%s not a supported filter method' % chosen
-                # Disable those methods not chosen.
-                for m in methods:
-                    if m != chosen:
-                        # setdefault = Don't override actual methods the
-                        # class has in fact provided itself.
-                        if not m in klass.__dict__:
-                            setattr(klass, m, None)
-
-            return klass
-
     def open(self, out, source_path, **kw):
         self._evaluate([out, source_path], kw, out)
 
@@ -418,13 +444,13 @@ class ExternalTool(Filter):
             def replace(arg):
                 try:
                     return arg.format(*args, **kwargs)
-                except KeyError, e:
+                except KeyError as e:
                     # Treat "output" and "input" variables special, they
                     # are dealt with in :meth:`subprocess` instead.
                     if e.args[0] not in ('input', 'output'):
                         raise
                     return arg
-            argv = map(replace, self.argv)
+            argv = list(map(replace, self.argv))
         else:
             argv = self.argv
         self.subprocess(argv, out, data=data)
@@ -449,8 +475,10 @@ class ExternalTool(Filter):
         class tempfile_on_demand(object):
             def __repr__(self):
                 if not hasattr(self, 'filename'):
-                    self.fd, self.filename = tempfile.mkstemp()
+                    fd, self.filename = tempfile.mkstemp()
+                    os.close(fd)
                 return self.filename
+
             @property
             def created(self):
                 return hasattr(self, 'filename')
@@ -459,39 +487,46 @@ class ExternalTool(Filter):
         input_file = tempfile_on_demand()
         output_file = tempfile_on_demand()
         if hasattr(str, 'format'):   # Support Python 2.5 without the feature
-            argv = map(lambda item:
-                item.format(input=input_file, output=output_file), argv)
+            argv = list(map(lambda item:
+                       item.format(input=input_file, output=output_file), argv))
 
         try:
+            data = (data.read() if hasattr(data, 'read') else data)
+            if data is not None:
+                data = data.encode('utf-8')
+
             if input_file.created:
-                if not data:
+                if data is None:
                     raise ValueError(
                         '{input} placeholder given, but no data passed')
-                with os.fdopen(input_file.fd, 'wb') as f:
-                    f.write(data.read() if hasattr(data, 'read') else data)
+                with open(input_file.filename, 'wb') as f:
+                    f.write(data)
                     # No longer pass to stdin
                     data = None
-
-            proc = subprocess.Popen(
-                argv,
-                # we cannot use the in/out streams directly, as they might be
-                # StringIO objects (which are not supported by subprocess)
-                stdout=subprocess.PIPE,
-                stdin=subprocess.PIPE,
-                stderr=subprocess.PIPE)
-            stdout, stderr = proc.communicate(
-                data.read() if hasattr(data, 'read') else data)
+            try:
+                proc = subprocess.Popen(
+                    argv,
+                    # we cannot use the in/out streams directly, as they might be
+                    # StringIO objects (which are not supported by subprocess)
+                    stdout=subprocess.PIPE,
+                    stdin=subprocess.PIPE,
+                    stderr=subprocess.PIPE,
+                    shell=os.name == 'nt')
+            except OSError:
+                raise FilterError('Program file not found: %s.' % argv[0])
+            stdout, stderr = proc.communicate(data)
             if proc.returncode:
                 raise FilterError(
                     '%s: subprocess returned a non-success result code: '
                     '%s, stdout=%s, stderr=%s' % (
-                        cls.name or cls.__name__, proc.returncode, stdout, stderr))
+                        cls.name or cls.__name__, 
+                        proc.returncode, stdout, stderr))
             else:
                 if output_file.created:
-                    with os.fdopen(output_file.fd, 'rb') as f:
-                        out.write(f.read())
+                    with open(output_file.filename, 'rb') as f:
+                        out.write(f.read().decode('utf-8'))
                 else:
-                    out.write(stdout)
+                    out.write(stdout.decode('utf-8'))
         finally:
             if output_file.created:
                 os.unlink(output_file.filename)
@@ -529,6 +564,7 @@ class JavaTool(ExternalTool):
 
 _FILTERS = {}
 
+
 def register_filter(f):
     """Add the given filter to the list of know filters.
     """
@@ -552,7 +588,7 @@ def get_filter(f, *args, **kwargs):
         # Don't need to do anything.
         assert not args and not kwargs
         return f
-    elif isinstance(f, basestring):
+    elif isinstance(f, six.string_types):
         if f in _FILTERS:
             klass = _FILTERS[f]
         else:
@@ -567,24 +603,74 @@ def get_filter(f, *args, **kwargs):
 
     return klass(*args, **kwargs)
 
+CODE_FILES = ['.py', '.pyc', '.so']
+
+
+def is_module(name):
+    """Is this a recognized module type?
+    
+    Does this name end in one of the recognized CODE_FILES extensions?
+    
+    The file is assumed to exist, as unique_modules has found it using 
+    an os.listdir() call.
+    
+    returns the name with the extension stripped (the module name) or 
+        None if the name does not appear to be a module
+    """
+    for ext in CODE_FILES:
+        if name.endswith(ext):
+            return name[:-len(ext)]
+
+
+def is_package(directory):
+    """Is the (fully qualified) directory a python package?
+    
+    """
+    for ext in ['.py', '.pyc']:
+        if os.path.exists(os.path.join(directory, '__init__'+ext)):
+            return True 
+
+
+def unique_modules(directory):
+    """Find all unique module names within a directory 
+    
+    For each entry in the directory, check if it is a source 
+    code file-type (using is_code(entry)), or a directory with 
+    a source-code file-type at entry/__init__.py[c]?
+    
+    Filter the results to only produce a single entry for each 
+    module name.
+    
+    Filter the results to not include '_' prefixed names.
+    
+    yields each entry as it is encountered
+    """
+    found = {}
+    for entry in sorted(os.listdir(directory)):
+        if entry.startswith('_'):
+            continue 
+        module = is_module(entry)
+        if module:
+            if module not in found:
+                found[module] = entry
+                yield module
+        elif is_package(os.path.join(directory, entry)):
+            if entry not in found:
+                found[entry] = entry 
+                yield entry 
+
 
 def load_builtin_filters():
     from os import path
     import warnings
 
     current_dir = path.dirname(__file__)
-    for entry in os.listdir(current_dir):
-        if entry.endswith('.py'):
-            name = path.splitext(entry)[0]
-        elif path.exists(path.join(current_dir, entry, '__init__.py')):
-            name = entry
-        else:
-            continue
+    for name in unique_modules(current_dir):
 
         module_name = 'webassets.filter.%s' % name
         try:
             module = import_module(module_name)
-        except Exception, e:
+        except Exception as e:
             warnings.warn('Error while loading builtin filter '
                           'module \'%s\': %s' % (module_name, e))
         else:
@@ -597,4 +683,3 @@ def load_builtin_filters():
                         continue
                     register_filter(attr)
 load_builtin_filters()
-
