@@ -7,10 +7,15 @@ from __future__ import with_statement
 
 import copy
 from os import path
-import urllib2
-from StringIO import StringIO
+try:
+    from urllib.request import \
+        HTTPHandler, build_opener, install_opener, addinfourl
+except ImportError: # Py2
+    from urllib2 import HTTPHandler, build_opener, install_opener, addinfourl
+from webassets.six import StringIO
+from webassets.six.moves import filter
 
-from nose.tools import assert_raises, assert_equals
+from nose.tools import assert_raises, assert_equal
 from nose import SkipTest
 
 from webassets import Bundle
@@ -22,8 +27,8 @@ from webassets.filter import Filter
 from webassets.updater import TimestampUpdater, SKIP_CACHE
 from webassets.version import Manifest, Version, VersionIndeterminableError
 
-from helpers import (
-    TempEnvironmentHelper, assert_raises_regexp)
+from .helpers import (
+    TempEnvironmentHelper, assert_raises_regex)
 
 
 class TestBundleConfig(TempEnvironmentHelper):
@@ -35,7 +40,7 @@ class TestBundleConfig(TempEnvironmentHelper):
         """
         try:
             Bundle(yaddayada=True)
-        except TypeError, e:
+        except TypeError as e:
             assert "unexpected keyword argument" in ("%s" % e)
         else:
             raise Exception('Expected TypeError not raised')
@@ -55,6 +60,16 @@ class TestBundleConfig(TempEnvironmentHelper):
         bundle.extra = None
         assert Bundle(bundle).extra == {}
 
+    def test_post_init_set_debug_to_True(self):
+        bundle = Bundle()
+        bundle.debug = True
+        assert bundle.debug is True
+
+    def test_post_init_set_debug_to_False(self):
+        bundle = Bundle()
+        bundle.debug = False
+        assert bundle.debug is False
+
     def test_filter_assign(self):
         """Test the different ways we can assign filters to the bundle.
         """
@@ -138,7 +153,7 @@ class TestBundleConfig(TempEnvironmentHelper):
 class DummyVersion(Version):
     def __init__(self, version=None):
         self.version = version
-    def determine_version(self, bundle, env, hunk=None):
+    def determine_version(self, bundle, ctx, hunk=None):
         if not self.version:
             raise VersionIndeterminableError('dummy has no version')
         return self.version
@@ -147,7 +162,7 @@ class DummyManifest(Manifest):
     def __init__(self, version=None):
         self.log = []
         self.version = version
-    def query(self, bundle, env):
+    def query(self, bundle, ctx):
         return self.version
     def remember(self, *a, **kw):
         self.log.append((a, kw))
@@ -233,9 +248,9 @@ class TestVersionFeatures(TempEnvironmentHelper):
         bundle = self.mkbundle('in', output='out-%(version)s')
         self.env.manifest.version = None
         self.env.versions.version = None
-        assert_raises_regexp(
+        assert_raises_regex(
             BundleError, 'dummy has no version', bundle.get_version)
-        assert_raises_regexp(
+        assert_raises_regex(
             BundleError, 'dummy has no version', bundle.resolve_output)
 
     def test_get_version_refresh(self):
@@ -472,7 +487,7 @@ class TestGlobbing(TempEnvironmentHelper):
         self.env.debug = True
         urls = self.mkbundle('*.js', output='out').urls()
         urls.sort()
-        assert_equals(urls, ['/file1.js', '/file2.js'])
+        assert_equal(urls, ['/file1.js', '/file2.js'])
 
     def test_empty_pattern(self):
         bundle = self.mkbundle('*.xyz', output='out')
@@ -507,18 +522,18 @@ class TestGlobbing(TempEnvironmentHelper):
         """[Regression] Glob should be smart enough not to pick
         up directories."""
         self.create_directories('subdir')
-        assert not filter(lambda s: 'subdir' in s,
-                           get_all_bundle_files(self.mkbundle('*')))
+        assert not list(filter(lambda s: 'subdir' in s,
+                           get_all_bundle_files(self.mkbundle('*'))))
 
     def test_glob_exclude_output(self):
         """Never include the output file in the globbinb result.
         """
         self.create_files(['out.js'])
-        assert not filter(lambda s: 'out.js' in s,
-            get_all_bundle_files(self.mkbundle('*', output='out.js')))
+        assert not list(filter(lambda s: 'out.js' in s,
+            get_all_bundle_files(self.mkbundle('*', output='out.js'))))
 
 
-class MockHTTPHandler(urllib2.HTTPHandler):
+class MockHTTPHandler(HTTPHandler):
 
     def __init__(self, urls={}):
         self.urls = urls
@@ -528,11 +543,11 @@ class MockHTTPHandler(urllib2.HTTPHandler):
         try:
             content = self.urls[url]
         except KeyError:
-            resp = urllib2.addinfourl(StringIO(""), None, url)
+            resp = addinfourl(StringIO(""), None, url)
             resp.code = 404
             resp.msg = "OK"
         else:
-            resp = urllib2.addinfourl(StringIO(content), None, url)
+            resp = addinfourl(StringIO(content), None, url)
             resp.code = 200
             resp.msg = "OK"
         return resp
@@ -544,9 +559,9 @@ class TestUrlContents(TempEnvironmentHelper):
 
     def setup(self):
         TempEnvironmentHelper.setup(self)
-        mock_opener = urllib2.build_opener(MockHTTPHandler({
-            'http://foo': 'function() {}'}))
-        urllib2.install_opener(mock_opener)
+        mock_opener = build_opener(MockHTTPHandler({
+            'http://foo': u'function() {}'}))
+        install_opener(mock_opener)
 
     def test_valid_url(self):
         self.mkbundle('http://foo', output='out').build()
@@ -587,9 +602,9 @@ class TestResolverAPI(TempEnvironmentHelper):
         """Test the method is properly used in the build process.
         """
         class MyResolver(self.env.resolver_class):
-            def resolve_source(self, item):
-                return path.join(self.env.directory, 'foo')
-        self.env.resolver = MyResolver(self.env)
+            def resolve_source(self, ctx, item):
+                return path.join(ctx.directory, 'foo')
+        self.env.resolver = MyResolver()
 
         self.create_files({'foo': 'foo'})
         self.mkbundle('bar', output='out').build()
@@ -599,9 +614,9 @@ class TestResolverAPI(TempEnvironmentHelper):
         """The bundle dependencies also go through normalization.
         """
         class MyResolver(self.env.resolver_class):
-            def resolve_source(self, item):
-                return path.join(self.env.directory, item[::-1])
-        self.env.resolver = MyResolver(self.env)
+            def resolve_source(self, ctx, item):
+                return path.join(ctx.directory, item[::-1])
+        self.env.resolver = MyResolver()
 
         self.create_files(['foo', 'dep', 'out'])
         b = self.mkbundle('oof', depends=('ped',), output='out')
@@ -621,9 +636,9 @@ class TestResolverAPI(TempEnvironmentHelper):
         See https://github.com/miracle2k/webassets/issues/71
         """
         class MyResolver(self.env.resolver_class):
-            def resolve_source(self, item):
-                return path.join(self.env.directory, (".".join(item)))
-        self.env.resolver = MyResolver(self.env)
+            def resolve_source(self, ctx, item):
+                return path.join(ctx.directory, (".".join(item)))
+        self.env.resolver = MyResolver()
 
         self.create_files({'foo.css': 'foo'})
         bundle = self.mkbundle(('foo', 'css'), output='out')
@@ -649,7 +664,7 @@ class TestResolverAPI(TempEnvironmentHelper):
         This is so that implementations can apply url encoding without
         worrying.
         """
-        def dummy(url):
+        def dummy(ctx, url):
             return url % {'version': 'not the correct version'}
         self.env.resolver.resolve_output_to_url = dummy
         bundle = self.mkbundle('in', output='out-%(version)s')
