@@ -2,12 +2,11 @@ from __future__ import with_statement
 import sys
 from nose.tools import assert_raises
 import textwrap
-from StringIO import StringIO
+from webassets.env import Environment
+from webassets.utils import StringIO
 from webassets.bundle import Bundle
 from webassets.loaders import PythonLoader, YAMLLoader, LoaderError
-from webassets.exceptions import ImminentDeprecationWarning
 from nose import SkipTest
-from helpers import check_warnings
 
 
 class TestYAML(object):
@@ -27,7 +26,7 @@ class TestYAML(object):
     def test_load_bundles(self):
         bundles = self.loader("""
         standard:
-            filters: cssmin,gzip
+            filters: cssmin,jsmin
             output: output.css
             contents:
                 - file1
@@ -46,6 +45,8 @@ class TestYAML(object):
                     - lessfile2
                     - contents:
                         reallynested.css
+                      config:
+                        closure_bin: /tmp/closure
                     - lessfile3
                 
         """).load_bundles()
@@ -63,18 +64,19 @@ class TestYAML(object):
         assert len(nested_bundle.filters) == 1
         assert len(nested_bundle.contents) == 4
         assert isinstance(nested_bundle.contents[2], Bundle)
+        assert nested_bundle.contents[2].config['closure_bin'] == '/tmp/closure'
 
     def test_load_recursive_bundles(self):
         bundles = self.loader("""
         standard:
-            filters: cssmin,gzip
+            filters: cssmin,jsmin
             output: output.css
             contents:
                 - file1
                 - file2
         recursive:
             output: recursive.css
-            filters: cssmin
+            filters: jsmin
             contents:
                 - cssfile1
                 - standard
@@ -123,15 +125,6 @@ class TestYAML(object):
         """
         self.loader("""foo: bar""").load_environment()
 
-    def test_load_deprecated_attrs(self):
-        with check_warnings(("", ImminentDeprecationWarning)) as w:
-            environment = self.loader("""
-            url: /foo
-            directory: something
-            expire: false
-            """).load_environment()
-            assert environment.url_expire == False
-
     def test_load_environment_directory_base(self):
         environment = self.loader("""
         url: /foo
@@ -169,5 +162,27 @@ class TestPython(object):
         loader = PythonLoader(module)
         bundles = loader.load_bundles()
         assert len(bundles) == 1
-        assert bundles.values()[0].contents[0] == 'bar'
+        assert list(bundles.values())[0].contents[0] == 'bar'
 
+    def test_load_environment_with_prefix(self):
+        import types
+        module = types.ModuleType("testing")
+        module2 = types.ModuleType("testing2")
+        module.environment = Environment() # default name
+        module2.assets = Environment()
+        sys.modules["testing"] = module
+        sys.modules["testing2"] = module2
+
+        loader = PythonLoader("testing")
+        env = loader.load_environment()
+        assert env == module.environment
+
+        loader2 = PythonLoader("testing:environment")
+        assert loader2.environment == "environment"
+        env2 = loader2.load_environment()
+        assert env2 == module.environment
+
+        loader3 = PythonLoader("testing2:assets")
+        assert loader3.environment == "assets"
+        env3 = loader3.load_environment()
+        assert env3 == module2.assets
